@@ -9,21 +9,22 @@ import re
 
 from sqlalchemy import (Enum, ForeignKeyConstraint, PrimaryKeyConstraint, CheckConstraint, UniqueConstraint, Table,
                         Column)
+from sqlalchemy.schema import ForeignKey
 from sqlalchemy.util import OrderedDict
 from sqlalchemy.types import Boolean, String
 import sqlalchemy
 
 try:
-    from sqlalchemy.sql.expression import TextClause
+    from sqlalchemy.sql.expression import text, TextClause
 except ImportError:
     # SQLAlchemy < 0.8
-    from sqlalchemy.sql.expression import _TextClause as TextClause
-
+    from sqlalchemy.sql.expression import text, _TextClause as TextClause
 
 _re_boolean_check_constraint = re.compile(r"(?:(?:.*?)\.)?(.*?) IN \(0, 1\)")
 _re_column_name = re.compile(r'(?:(["`]?)(?:.*)\1\.)?(["`]?)(.*)\2')
 _re_enum_check_constraint = re.compile(r"(?:(?:.*?)\.)?(.*?) IN \((.+)\)")
 _re_enum_item = re.compile(r"'(.*?)(?<!\\)'")
+_re_invalid_identifier = re.compile(r'[^a-zA-Z0-9_]' if sys.version_info[0] < 3 else r'(?u)\W')
 
 
 class _DummyInflectEngine(object):
@@ -31,6 +32,13 @@ class _DummyInflectEngine(object):
         return noun
 
 
+def _convert_to_valid_identifier(name):
+    assert name, 'Identifier cannot be empty'
+    if name[0].isdigit() or iskeyword(name):
+        name = '_' + name
+    return _re_invalid_identifier.sub('_', name)
+
+
 def _get_compiled_expression(statement):
     """Returns the statement in a form where any placeholders have been filled in."""
     if isinstance(statement, TextClause):
@@ -109,15 +117,6 @@ def _render_column_type(coltype):
     return text
 
 
-def _render_fk(fk):
-    text = "ForeignKey('%s.%s'" % (fk.column.table.fullname, fk.column.name)
-    if fk.deferrable:
-        text += ', deferrable=True'
-    if fk.initially:
-        text += ', initially=%r' % fk.initially
-    return text + ')'
-
-
 def _render_column(column, show_name):
     kwarg = []
     is_sole_pk = column.primary_key and len(column.table.primary_key) == 1
@@ -142,24 +141,41 @@ def _render_column(column, show_name):
     elif has_index:
         column.index = True
         kwarg.append('index')
-    if column.server_default and not is_sole_pk:
-        column.server_default = _get_compiled_expression(column.server_default.arg)
-        kwarg.append('server_default')
+    if column.server_default:
+        default_expr = _get_compiled_expression(column.server_default.arg)
+        if '\n' in default_expr:
+            server_default = 'server_default=text("""\\\n{0}""")'.format(default_expr)
+        else:
+            server_default = 'server_default=text("{0}")'.format(default_expr)
 
     return 'Column({0})'.format(', '.join(
         ([repr(column.name)] if show_name else []) +
         ([_render_column_type(column.type)] if render_coltype else []) +
-        [_render_fk(x) for x in dedicated_fks] +
+        [_render_constraint(x) for x in dedicated_fks] +
         [repr(x) for x in column.constraints] +
-        ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg]))
+        ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg] +
+        ([server_default] if column.server_default else [])
+    ))
 
 
 def _render_constraint(constraint):
-    if isinstance(constraint, ForeignKeyConstraint):
+    def render_fk_options(*opts):
+        opts = [repr(opt) for opt in opts]
+        for attr in 'ondelete', 'onupdate', 'deferrable', 'initially', 'match':
+            value = getattr(constraint, attr, None)
+            if value:
+                opts.append('{0}={1!r}'.format(attr, value))
+
+        return ', '.join(opts)
+
+    if isinstance(constraint, ForeignKey):
+        remote_column = '{0}.{1}'.format(constraint.column.table.fullname, constraint.column.name)
+        return 'ForeignKey({0})'.format(render_fk_options(remote_column))
+    elif isinstance(constraint, ForeignKeyConstraint):
         local_columns = constraint.columns
         remote_columns = ['{0}.{1}'.format(fk.column.table.fullname, fk.column.name)
                           for fk in constraint.elements]
-        return 'ForeignKeyConstraint({0!r}, {1!r})'.format(local_columns, remote_columns)
+        return 'ForeignKeyConstraint({0})'.format(render_fk_options(local_columns, remote_columns))
     elif isinstance(constraint, CheckConstraint):
         return 'CheckConstraint({0!r})'.format(_get_compiled_expression(constraint.sqltext))
     elif isinstance(constraint, UniqueConstraint):
@@ -168,8 +184,10 @@ def _render_constraint(constraint):
 
 
 def _render_index(index):
-    columns = [repr(col.name) for col in index.columns]
-    return 'Index({0!r}, {1})'.format(index.name, ', '.join(columns))
+    extra_args = [repr(col.name) for col in index.columns]
+    if index.unique:
+        extra_args.append('unique=True')
+    return 'Index({0!r}, {1})'.format(index.name, ', '.join(extra_args))
 
 
 class ImportCollector(OrderedDict):
@@ -210,6 +228,8 @@ class Model(object):
 
         for column in self.table.columns:
             collector.add_import(column.type)
+            if column.server_default:
+                collector.add_literal_import('sqlalchemy', 'text')
 
         for constraint in sorted(self.table.constraints, key=_get_constraint_sort_key):
             if isinstance(constraint, ForeignKeyConstraint):
@@ -267,8 +287,7 @@ class ModelClass(Model):
 
         # Assign attribute names for columns
         for column in table.columns:
-            attrname = column.name + '_' if iskeyword(column.name) else column.name
-            self.attributes[attrname] = column
+            self._add_attribute(column.name, column)
 
         # Add many-to-one relationships
         pk_column_names = set(col.name for col in table.primary_key.columns)
@@ -278,25 +297,31 @@ class ModelClass(Model):
                 if detect_joined and self.parent_name == 'Base' and set(constraint.columns) == pk_column_names:
                     self.parent_name = target_cls
                 else:
-                    self._add_relationship(ManyToOneRelationship(self.name, target_cls, constraint, inflect_engine))
+                    relationship_ = ManyToOneRelationship(self.name, target_cls, constraint, inflect_engine)
+                    self._add_attribute(relationship_.preferred_name, relationship_)
 
         # Add many-to-many relationships
         for association_table in association_tables:
             fk_constraints = [c for c in association_table.constraints if isinstance(c, ForeignKeyConstraint)]
             fk_constraints.sort(key=_get_constraint_sort_key)
             target_cls = self._tablename_to_classname(fk_constraints[1].elements[0].column.table.name, inflect_engine)
-            self._add_relationship(ManyToManyRelationship(self.name, target_cls, association_table))
+            relationship_ = ManyToManyRelationship(self.name, target_cls, association_table)
+            self._add_attribute(relationship_.preferred_name, relationship_)
 
     @staticmethod
     def _tablename_to_classname(tablename, inflect_engine):
-        camel_case_name = ''.join(part.capitalize() for part in tablename.split('_'))
+        camel_case_name = ''.join(part[:1].upper() + part[1:] for part in tablename.split('_'))
         return inflect_engine.singular_noun(camel_case_name) or camel_case_name
 
-    def _add_relationship(self, relationship):
-        for attrname in relationship.suggested_names:
-            if attrname not in self.attributes and not iskeyword(attrname):
-                self.attributes[attrname] = relationship
-                break
+    def _add_attribute(self, attrname, value):
+        attrname = tempname = _convert_to_valid_identifier(attrname)
+        counter = 1
+        while tempname in self.attributes:
+            tempname = attrname + str(counter)
+            counter += 1
+
+        self.attributes[tempname] = value
+        return tempname
 
     def add_imports(self, collector):
         super(ModelClass, self).add_imports(collector)
@@ -366,15 +391,6 @@ class Relationship(object):
         self.target_cls = target_cls
         self.kwargs = OrderedDict()
 
-    @property
-    def suggested_names(self):
-        yield self.preferred_name if not iskeyword(self.preferred_name) else self.preferred_name + '_'
-
-        iteration = 0
-        while True:
-            iteration += 1
-            yield self.preferred_name + str(iteration)
-
     def render(self):
         text = 'relationship('
         args = [repr(self.target_cls)]
@@ -465,7 +481,7 @@ class CodeGenerator(object):
         for table in metadata.tables.values():
             # Link tables have exactly two foreign key constraints and all columns are involved in them
             fk_constraints = [constr for constr in table.constraints if isinstance(constr, ForeignKeyConstraint)]
-            if (len(fk_constraints) == 2 and all(col.foreign_keys for col in table.columns)):
+            if len(fk_constraints) == 2 and all(col.foreign_keys for col in table.columns):
                 association_tables.add(table.name)
                 tablename = sorted(fk_constraints, key=_get_constraint_sort_key)[0].elements[0].column.table.name
                 links[tablename].append(table)
