@@ -6,6 +6,7 @@ import re
 from sqlalchemy.engine import create_engine
 from sqlalchemy.schema import (MetaData, Table, Column, CheckConstraint, UniqueConstraint, Index, ForeignKey,
                                ForeignKeyConstraint)
+from sqlalchemy.sql.expression import text
 from sqlalchemy.types import INTEGER, SMALLINT, VARCHAR, NUMERIC
 from sqlalchemy.dialects.postgresql.base import BIGINT, DOUBLE_PRECISION, BOOLEAN, ENUM
 from sqlalchemy.dialects.mysql.base import TINYINT
@@ -258,7 +259,7 @@ t_simple_items = Table(
             Column('text', VARCHAR)
         )
         simple_items.indexes.add(Index('idx_number', simple_items.c.number))
-        simple_items.indexes.add(Index('idx_text_number', simple_items.c.text, simple_items.c.number))
+        simple_items.indexes.add(Index('idx_text_number', simple_items.c.text, simple_items.c.number, unique=True))
         simple_items.indexes.add(Index('idx_text', simple_items.c.text, unique=True))
 
         assert self.generate_code() == """\
@@ -274,7 +275,7 @@ t_simple_items = Table(
     Column('id', Integer),
     Column('number', Integer, index=True),
     Column('text', String, unique=True),
-    Index('idx_text_number', 'text', 'number')
+    Index('idx_text_number', 'text', 'number', unique=True)
 )
 """
 
@@ -414,7 +415,8 @@ class SimpleItem(Base):
             Column('id', INTEGER, primary_key=True),
             Column('container_id1', INTEGER),
             Column('container_id2', INTEGER),
-            ForeignKeyConstraint(['container_id1', 'container_id2'], ['simple_containers.id1', 'simple_containers.id2'])
+            ForeignKeyConstraint(['container_id1', 'container_id2'], ['simple_containers.id1', 'simple_containers.id2'],
+                                 ondelete='CASCADE', onupdate='CASCADE')
         )
         Table(
             'simple_containers', self.metadata,
@@ -443,7 +445,8 @@ class SimpleContainer(Base):
 class SimpleItem(Base):
     __tablename__ = 'simple_items'
     __table_args__ = (
-        ForeignKeyConstraint(['container_id1', 'container_id2'], ['simple_containers.id1', 'simple_containers.id2']),
+        ForeignKeyConstraint(['container_id1', 'container_id2'], ['simple_containers.id1', 'simple_containers.id2'], \
+ondelete='CASCADE', onupdate='CASCADE'),
     )
 
     id = Column(Integer, primary_key=True)
@@ -899,7 +902,8 @@ t_simple_items = Table(
     def test_foreign_key_options(self):
         Table(
             'simple_items', self.metadata,
-            Column('name', VARCHAR, ForeignKey('simple_items.name', deferrable=True, initially='DEFERRED'))
+            Column('name', VARCHAR, ForeignKey('simple_items.name', ondelete='CASCADE', onupdate='CASCADE',
+                                               deferrable=True, initially='DEFERRED'))
         )
 
         assert self.generate_code() == """\
@@ -912,7 +916,8 @@ metadata = MetaData()
 
 t_simple_items = Table(
     'simple_items', metadata,
-    Column('name', String, ForeignKey('simple_items.name', deferrable=True, initially='DEFERRED'))
+    Column('name', String, ForeignKey('simple_items.name', ondelete='CASCADE', onupdate='CASCADE', \
+deferrable=True, initially='DEFERRED'))
 )
 """
 
@@ -955,3 +960,169 @@ class OtherItem(Base):
 
     id = Column(Integer, primary_key=True)
 """
+
+    def test_pk_default(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('id', INTEGER, primary_key=True, server_default=text('uuid_generate_v4()'))
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer, text
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+
+    id = Column(Integer, primary_key=True, server_default=text("uuid_generate_v4()"))
+"""
+
+    def test_server_default_multiline(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('id', INTEGER, primary_key=True, server_default=text("""\
+/*Comment*/
+/*Next line*/
+something()"""))
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer, text
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+
+    id = Column(Integer, primary_key=True, server_default=text(\"""\\
+/*Comment*/
+/*Next line*/
+something()\"""))
+"""
+
+    def test_invalid_attribute_names(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('id-test', INTEGER, primary_key=True),
+            Column('4test', INTEGER),
+            Column('_4test', INTEGER),
+            Column('def', INTEGER)
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+
+    id_test = Column('id-test', Integer, primary_key=True)
+    _4test = Column('4test', Integer)
+    _4test1 = Column('_4test', Integer)
+    _def = Column('def', Integer)
+"""
+
+    def test_pascal(self):
+        Table(
+            'CustomerAPIPreference', self.metadata,
+            Column('id', INTEGER, primary_key=True)
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class CustomerAPIPreference(Base):
+    __tablename__ = 'CustomerAPIPreference'
+
+    id = Column(Integer, primary_key=True)
+"""
+
+    def test_underscore(self):
+        Table(
+            'customer_api_preference', self.metadata,
+            Column('id', INTEGER, primary_key=True)
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class CustomerApiPreference(Base):
+    __tablename__ = 'customer_api_preference'
+
+    id = Column(Integer, primary_key=True)
+"""
+
+    def test_pascal_underscore(self):
+        Table(
+            'customer_API_Preference', self.metadata,
+            Column('id', INTEGER, primary_key=True)
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class CustomerAPIPreference(Base):
+    __tablename__ = 'customer_API_Preference'
+
+    id = Column(Integer, primary_key=True)
+"""
+
+    def test_pascal_multiple_underscore(self):
+        Table(
+            'customer_API__Preference', self.metadata,
+            Column('id', INTEGER, primary_key=True)
+        )
+
+        assert self.generate_code() == """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class CustomerAPIPreference(Base):
+    __tablename__ = 'customer_API__Preference'
+
+    id = Column(Integer, primary_key=True)
+"""
